//=============================================================================
// IonPlasma_Weapon
//=============================================================================

class F_BikeIonPlasma_Weapon extends FragEdSpecialBike_Weapons;

var()	class<FX_BikeLaserCannon_BeamFire> BeamEffectClass;
var()	class<FX_BikeLaserCannon_BeamFireRed> BeamEffectClassRed;
var		float	StartHoldTime, MaxHoldTime, ShockMomentum, ShockRadius;
var		bool	bHoldingFire, bFireMode;
var		int		BeamCount, OldBeamCount;

var	sound	ChargingSound, ShockSound;

var	FX_FBikeIonPlasma_AimLaser	AimLaser;
var()   float   MaxShieldHealth;
var()   float   MaxDelayTime;
var()   float   ShieldRechargeRate;
var		float	LastShieldHitTime;

var     float   CurrentShieldHealth;
var     float   CurrentDelayTime;
var     float   CurrentRechargeTime;
var     bool    bShieldActive, bLastShieldActive;
var		bool	bPutShieldUp;
var     byte    ShieldHitCount, LastShieldHitCount;

var     F_BikeShield   ShockShield;
replication
{
    reliable if (bNetOwner && Role == ROLE_Authority)
        CurrentShieldHealth;

    reliable if (Role == ROLE_Authority)
        bShieldActive, ShieldHitCount;

    reliable if ( bNetDirty && !bNetOwner && Role == ROLE_Authority )
		bFireMode, BeamCount;
}

simulated function PostNetBeginPlay()
{

    Super.PostNetBeginPlay();

    ShockShield = spawn(class'F_BikeShield', self);

    if (ShockShield != None)
        AttachToBone(ShockShield, 'LSite');
}



event bool AttemptFire(Controller C, bool bAltFire)
{
  	if (Role != ROLE_Authority || bForceCenterAim)
		return False;

	if (bAltFire)
    {
		if ( ShockShield != None )
		{
			CurrentDelayTime = 0;

			if (!bShieldActive && CurrentShieldHealth > 0)
			{
				ActivateShield();
			}
		}
    }
	else if ( (AIController(C) != None) && bShieldActive && (VSize(C.Target.Location - Instigator.Location) > 900) )
	{
		DeactivateShield();
	}

	if (FireCountdown <= 0)
	{
		CalcWeaponFire();
		if (bCorrectAim)
			WeaponFireRotation = AdjustAim(bAltFire);
		if (Spread > 0)
			WeaponFireRotation = rotator(vector(WeaponFireRotation) + VRand()*FRand()*Spread);

        DualFireOffset *= -1;

		Instigator.MakeNoise(1.0);
		if (!bAltFire)
		{
		    FireCountdown = FireInterval;
		    Fire(C);
		}
		AimLockReleaseTime = Level.TimeSeconds + FireCountdown * FireIntervalAimLock;

	    return True;
	}

	return False;
}

function CeaseAltFire()
{
    if (ShockShield != None)
        DeactivateShield();
}


simulated function ActivateShield()
{
    bShieldActive = true;

    if (ShockShield != None)
        ShockShield.ActivateShield(Team);
}

simulated function DeactivateShield()
{
    bShieldActive = false;

    if (ShockShield != None)
        ShockShield.DeactivateShield();
}

simulated function PostNetReceive()
{
    Super.PostNetReceive();

    if (bShieldActive != bLastShieldActive)
    {
        if (bShieldActive)
            ActivateShield();
        else
            DeactivateShield();

        bLastShieldActive = bShieldActive;
    }

    if (ShockShield != None && ShieldHitCount != LastShieldHitCount)
    {
        ShockShield.SpawnHitEffect(Team);

        LastShieldHitCount = ShieldHitCount;
    }
}

//do effects (muzzle flash, force feedback, etc) immediately for the weapon's owner (don't wait for replication)
simulated event OwnerEffects()
{
	if (!bIsRepeatingFF)
	{
		if (bIsAltFire)
			ClientPlayForceFeedback( AltFireForce );
		else
			ClientPlayForceFeedback( FireForce );
	}
    ShakeView();

	if (Role < ROLE_Authority)
	{
		if (!bIsAltFire)
            FireCountdown = FireInterval;

		AimLockReleaseTime = Level.TimeSeconds + FireCountdown * FireIntervalAimLock;

        if (!bIsAltFire)
            FlashMuzzleFlash();

		if (AmbientEffectEmitter != None)
			AmbientEffectEmitter.SetEmitterStatus(true);

        // Play firing noise
        if (!bAmbientFireSound)
        {
            if (bIsAltFire)
                PlaySound(AltFireSoundClass, SLOT_None, FireSoundVolume/255.0,, AltFireSoundRadius,, false);
            else
                PlaySound(FireSoundClass, SLOT_None, FireSoundVolume/255.0,, FireSoundRadius,, false);
        }
	}
}

function NotifyShieldHit(int Dam, Pawn instigatedBy)
{
    if (Pawn(Owner) != None && Pawn(Owner).Controller != None && ((InstigatedBy == None) || (InstigatedBy.Controller == None) || !InstigatedBy.Controller.SameTeamAs(Pawn(Owner).Controller)))
    {
		LastShieldHitTime = Level.TimeSeconds;
        CurrentShieldHealth -= Dam;
        ShieldHitCount++;
        ShockShield.SpawnHitEffect(Team);
    }
}

simulated function Tick(float DT)
{
    Super.Tick(DT);

    if (ShockShield == None || Role < ROLE_Authority)
        return;

    if (CurrentShieldHealth <= 0)                        // Ran out of shield energy so deactivate
        DeactivateShield();

    if (!bShieldActive && (CurrentShieldHealth < MaxShieldHealth))  // Shield is off and needs recharge
    {
        if (CurrentDelayTime < MaxDelayTime)           // Shield is in delay
            CurrentDelayTime += DT;
        else                                           // Shield is in recharge
        {
            CurrentShieldHealth += ShieldRechargeRate * DT;
            if (CurrentShieldHealth >= MaxShieldHealth)
                  CurrentShieldHealth = MaxShieldHealth;
        }
    }
}

simulated function float ChargeBar()
{
	if ( bHoldingFire )
		return (FMin(Level.TimeSeconds - StartHoldTime, MaxHoldTime) / MaxHoldTime);
	else
		return 0;


}

function byte BestMode()
{
	local bot B;

	if ( CurrentShieldHealth <= 0 )
		return 0;
	if ( Projectile(Instigator.Controller.Target) != None )
		return 1;

	B = Bot(Instigator.Controller);
	if ( (B == None) || (B.Enemy == None) )
		return 0;

	if ( bPutShieldUp || !B.EnemyVisible() )
	{
		LastShieldHitTime = Level.TimeSeconds;
		bPutShieldUp = false;
		return 1;
	}

	if ( VSize(B.Enemy.Location - Location) < 900 )
	{
		if ( bShieldActive )
			return 0;
		else
			return 1;
	}
	if ( bShieldActive && (Level.TimeSeconds - LastShieldHitTime < 2) )
	   return 1;
	else if ( B.Enemy != B.Target )
		return 0;
	else
	{
		// check if near friendly node, and between it and enemy
		if ( (B.Squad.SquadObjective != None) && (VSize(B.Pawn.Location - B.Squad.SquadObjective.Location) < 1000)
			&& ((Normal(B.Enemy.Location - B.Squad.SquadObjective.Location) dot Normal(B.Pawn.Location - B.Squad.SquadObjective.Location)) > 0.7) )
			return 1;

		// use shield if heavily damaged
		if ( B.Pawn.Health < 0.3 * B.Pawn.Default.Health )
			return 1;

		// use shield against heavy vehicles
		if ( (B.Enemy == B.Target) && (Vehicle(B.Enemy) != None) && Vehicle(B.Enemy).ImportantVehicle() && (B.Enemy.Controller != None)
			&& ((Vector(B.Enemy.Controller.Rotation) dot Normal(Instigator.Location - B.Enemy.Location)) > 0.9) )
			return 1;
	   return 0;
	}
}

function ShieldAgainstIncoming(optional Projectile P)
{
	if ( P != None )
	{
		if ( FireCountDown > (VSize(P.Location - Location) - 1100)/VSize(P.Velocity) )
			return;
		// put shield up if pointed in right direction
		if ( Level.Game.GameDifficulty < 5 )
		{
			CalcWeaponFire();
			if ( (Normal(P.Location - WeaponFireLocation) dot vector(WeaponFireRotation)) < 0.7 )
				return;
		}
		LastShieldHitTime = Level.TimeSeconds;
		bPutShieldUp = true;
		Instigator.Controller.FireWeaponAt(Instigator.Controller.Focus);
	}
	else if ( Instigator.Controller.Enemy != None )
	{
		if ( (FireCountDown > 0.2) && (FRand() < 0.6) )
			return;
		LastShieldHitTime = Level.TimeSeconds;
		bPutShieldUp = true;
		Instigator.Controller.FireWeaponAt(Instigator.Controller.Focus);
	}
}

simulated function Destroyed()
{
	KillLaserBeam();
     if (ShockShield != None)
        ShockShield.Destroy();
	super.Destroyed();
}

function PlayFiring()
{
	AmbientSound = None;
	PlaySound(sound'WeaponSounds.BExplosion5', SLOT_None, FireSoundVolume/255.0,,,, False);
}

function PlayChargeUp()
{
	AmbientSound = ChargingSound;
}

function PlayRelease()
{
	AmbientSound = None;
	PlaySound(sound'WeaponSounds.TranslocatorModuleRegeneration', SLOT_None, FireSoundVolume/255.0,,,, False);
}

simulated event FlashMuzzleFlash()
{
	super.FlashMuzzleFlash();

	if ( Level.NetMode != NM_DedicatedServer )
	{
		if ( !IsAltFire() && BeamCount != OldBeamCount )
		{
			OldBeamCount = BeamCount;
			PlayAnim('Fire', 0.5, 0);
			super.ShakeView();
		}
	}
}

simulated function bool IsAltFire()
{
	if ( Instigator.IsLocallyControlled() )
		return bIsAltFire;

	return bFireMode;
}

simulated function ShakeView()
{
	if ( IsAltFire() )
		super.ShakeView();
}



function SpawnBeamEffect(Vector Start, Rotator Dir, Vector HitLocation, Vector HitNormal, int ReflectNum)
{
     local FX_BikeLaserCannon_BeamFire Beam;
    local FX_BikeLaserCannon_BeamFireRed RedBeam;

    if ( Vehicle(Instigator).Team == 1 )
      {
       Beam = Spawn(BeamEffectClass,,, Start, Dir);
       if (ReflectNum != 0) Beam.Instigator = None; // prevents client side repositioning of beam start
       Beam.AimAt(HitLocation, HitNormal);
      }
    else
      {
       RedBeam = Spawn(BeamEffectClassRed,,, Start, Dir);
       if (ReflectNum != 0) RedBeam.Instigator = None; // prevents client side repositioning of beam start
       RedBeam.AimAt(HitLocation, HitNormal);
      }
}

function SpawnLaserBeam()
{
	CalcWeaponFire();
    AimLaser = Spawn(class'FX_FBikeIonPlasma_AimLaser', Self,, WeaponFireLocation, WeaponFireRotation);
}

function KillLaserBeam()
{
	if ( AimLaser != None )
	{
		AimLaser.Destroy();
		AimLaser = None;
	}
}

simulated function UpdateLaserBeamLocation( out vector Start, out vector HitLocation )
{
	local vector	HitNormal;
	local rotator	Dir;

	CalcWeaponFire();
	Start		= WeaponFireLocation;
	Dir			= WeaponFireRotation;
	HitLocation	= vect(0,0,0);
	SimulateTraceFire( Start, Dir, HitLocation, HitNormal );
	//log("UpdateLaserBeamLocation WFL:" @ WeaponFireLocation @ "WFR:" @ WeaponFireRotation
	//	@ "Start:" @ Start @ "HL:" @ HitLocation );
}



state ProjectileFireMode
{
	simulated function ClientStartFire(Controller C, bool bAltFire)
	{
		bFireMode	= bAltFire;
		bIsAltFire	= bAltFire;
		//log("ClientStartFire");
		if ( !bAltFire )
		{
			if ( Role < ROLE_Authority && FireCountdown <= 0 )
			{
				bHoldingFire	= true;
				StartHoldTime	= Level.TimeSeconds;
				SetTimer( MaxHoldTime, false );
			}
			else
				SetTimer( FireCountDown, false );	// synch to starthold matches server (done in timer)
			//super.ClientStartFire(C, bAltFire);


		}
		else
		{
			super.ClientStartFire(C, bAltFire);
		}
	}

	simulated function Timer()
	{
		if ( !bHoldingFire )
		{
			if ( Role < Role_Authority && !bFireMode )
			{
				bHoldingFire	= true;
				StartHoldTime	= Level.TimeSeconds;
				SetTimer( MaxHoldTime, false );
			}
			return;
		}

		bHoldingFire	= false;
		FireCountdown	= FireInterval;
		SetTimer(FireInterval, false);

		if ( Role == ROLE_Authority )
		{
			KillLaserBeam();
			CalcWeaponFire();
			FlashCount++;
			PlayFiring();

			if ( AmbientEffectEmitter != None )
				AmbientEffectEmitter.SetEmitterStatus( true );

			TraceFire(WeaponFireLocation, WeaponFireRotation);
		}
		BeamCount++;
		FlashMuzzleFlash();
	}

	simulated function ClientStopFire(Controller C, bool bWasAltFire)
	{
		//log("ClientStopFire");
		super.ClientStopFire(C, bWasAltFire);

		if ( bHoldingFire )
		{
			bHoldingFire	= false;
			FireCountdown	= FireInterval;
			ClientPlayForceFeedback("BioRifleFire");
		}
		SetTimer(0, false);
	}

	function Fire(Controller C)
	{
		NetUpdateTime = Level.TimeSeconds - 1;
		bFireMode = false;
		//log("Fire");
		if ( !bHoldingFire )
		{
			PlayChargeUp();
			StartHoldTime	= Level.TimeSeconds;
			bHoldingFire	= true;
			SetTimer( MaxHoldTime, false );
			SpawnLaserBeam();
		}
	}


	function CeaseFire(Controller C)
	{
		//log("CeaseFire");

		KillLaserBeam();

		if ( bHoldingFire )
		{
			if ( Role == Role_Authority )
				PlayRelease();

			SetTimer(0, false);
			bHoldingFire	= false;
			FireCountdown	= FireInterval;
		}
	}
	 function AltFire(Controller C)
	{
	 super.AltFire(C);
	 }
}

defaultproperties
{
     BeamEffectClass=Class'BattleBikes.FX_BikeLaserCannon_BeamFire'
     BeamEffectClassRed=Class'BattleBikes.FX_BikeLaserCannon_BeamFireRed'

     bReplicateAnimations=True
     MaxShieldHealth=2000.000000
     MaxDelayTime=2.000000
     ShieldRechargeRate=400.000000
     CurrentShieldHealth=800.000000
     bShowChargingBar=True
     RotateSound=Sound'ONSBPSounds.ShockTank.TurretHorizontal'
     bNetNotify=True
     YawBone="Object01"
     PitchBone="Object02"
     PitchUpLimit=6000
     PitchDownLimit=61500
     WeaponFireAttachmentBone="FirePoint"
     WeaponFireOffset=32.000000
     RotationsPerSecond=0.180000
     Spread=0.015000
     MaxHoldTime=2.000000
     ChargingSound=Sound'AssaultSounds.AssaultRifle.IonPowerUp'
     ShockSound=Sound'ONSVehicleSounds-S.AVRiL.AvrilFire01'
     RedSkin=Texture'BattleBike_Tex.Bike78SkinA'
     BlueSkin=Texture'BattleBike_Tex.Bike78SkinABlue'
     FireInterval=1.500000
     EffectEmitterClass=None
     FireSoundClass=Sound'WeaponSounds.BaseImpactAndExplosions.BExplosion5'
     DamageMin=45
     DamageMax=45
     TraceRange=20000.000000
     ProjectileClass=None
     AIInfo(0)=(WarnTargetPct=0.990000,RefireRate=0.990000)
     AIInfo(1)=(bInstantHit=True,RefireRate=0.990000)
     Mesh=SkeletalMesh'BattleBikes_Anim.BikeIonCannonMesh'
     bForceSkelUpdate=True
     SoundPitch=112
     SoundRadius=512.000000
}
