//-----------------------------------------------------------
// FragEdSpecial Motorcycle with Rockets
//-----------------------------------------------------------
class FragEdSpecial extends ONSWheeledCraft;
#exec OBJ LOAD FILE=..\textures\BattleBike_Tex.utx
#exec OBJ LOAD FILE=..\Sounds\BattleBike_Snds.uax
#exec OBJ LOAD FILE=..\Animations\BattleBikes_Anim.ukx

//     Textures

var() float LocalRollDamping;
var() float LocalPitchDamping;
var() float LocalTurnDamping;
var() float AntiRollStiffness;
var() float BikeTurnDamping;

// Internal Vars

var bool ClientDoBikeJump; // Set to true while pending jump boost
var float BikeJumpForce; // How much force to apply per-tick during jump boost

var bool ReallyInAir;
var()   float   JumpDuration;
var()	float	JumpForceMag;
var     float   JumpCountdown;
var     float	JumpDelay, LastJumpTime;
var		bool							DoBikeJump;
var		bool							OldDoBikeJump;

// BoneRotations for Setting up look of Driving Cycle
var rotator ArmDriveL,ArmDriveR;
Var rotator NeckDrive;
Var rotator ThighDriveL,ThighDriveR;
Var rotator CalfDriveL,CalfDriveR;
//// TEAM STATUS ////
var()               Material        RedSkinB;
var()               Material        BlueSkinB;
var()class<FragEdSpecialBike_Weapons>BikeWeaponClass;
var() bool bHeadLight;
var() bool bHeadLightProjector;
replication
{
	reliable if( Role == ROLE_Authority)
		ClientDoBikeJump;

}

function Pawn CheckForHeadShot(Vector loc, Vector ray, float AdditionalScale)
{
    local vector X, Y, Z, newray;

    GetAxes(Rotation,X,Y,Z);

    if (Driver != None)
    {
        // Remove the Z component of the ray
        newray = ray;
        newray.Z = 0;
        if (abs(newray dot X) < 0.7 && Driver.IsHeadShot(loc, ray, AdditionalScale))
            return Driver;
    }

    return None;
}

simulated event TeamChanged()
{
     Super.TeamChanged();

    if (Team == 0 && RedSkin != None)
       {
        Skins[0] = RedSkin;
        Skins[1] = RedSkinB;
       }
    else if (Team == 1 && BlueSkin != None)
            {
             Skins[0] = BlueSkin;
             Skins[1] = BlueSkinB;
            }
}

function bool Dodge(eDoubleClickDir DoubleClickMove)
{
	Rise = 1;
	return true;
}
simulated event DrivingStatusChanged()
{
	Super.DrivingStatusChanged();
    JumpCountDown = 0.0;
}

simulated event KApplyForce(out vector Force, out vector Torque)
{
	local KRigidBodyState rbState;
	local Vector AngVel;
	local Vector RotDamping;
	local float RollOffset;
	local float AntiRollTorque;
	local Vector X,Y,Z;

	//Get Axes for "Calculations"
	GetAxes(Rotation, X,Y,Z);

	Super.KApplyForce(Force, Torque); // apply other forces first

	if (bDriving && JumpCountdown > 0.0)
	{
		Force += vect(0,0,1) * JumpForceMag;
	}
	if(ClientDoBikeJump)
	{
		// add forces to any existing forces already being applied, DO NOT OVERWRITE PREVIOUS FORCE VALUES
		Force += Z * BikeJumpForce;
		if(bVehicleOnGround)
			Torque += vect(0,0,1) * Steering * -AirTurnTorque;
		ClientDoBikeJump=false;
	}


	//Rotation Damping
	if(!ReallyInAir)
	{
		// Get body angular velocity in global space
		KGetRigidBodyState(rbState);
		AngVel.X = rbState.AngVel.X;
		AngVel.Y = rbState.AngVel.Y;
		AngVel.Z = rbState.AngVel.Z;

		//Local Damping - Note: currently this is global
		RotDamping.X = AngVel.X * LocalRollDamping;
		RotDamping.Y = AngVel.Y * LocalPitchDamping;
		RotDamping.Z = AngVel.Z * LocalTurnDamping;

		Torque -= RotDamping;


		//Turn damping - because TurnDamping doesn't seem to work
		Torque.Z -= AngVel.Z * BikeTurnDamping;


		//AntiRoll Stiffness
		RollOffset = Rotation.Roll / 32768.0;
		AntiRollTorque = RollOffset * AntiRollStiffness * 10;
		Torque += AntiRollTorque * X;
	}
}
simulated function CheckJumpDuck()
{
	local Emitter JumpEffect;

	// If we are on the ground, and press Rise, and we not currently in the middle of a jump, start a new one.
    if (JumpCountdown <= 0.0 && Rise > 0 && Level.TimeSeconds - JumpDelay >= LastJumpTime)
    {
        PlaySound(JumpSound,,1.0);

        if (Role == ROLE_Authority)
    	   DoBikeJump = !DoBikeJump;
           if(!ClientDoBikeJump)
		       ClientDoBikeJump = true;

        if(Level.NetMode != NM_DedicatedServer)
        {
            JumpEffect = Spawn(class'BikeJumpEffect');
            JumpEffect.SetBase(Self);

        }

    	if ( AIController(Controller) != None )
    		Rise = 0;

    	LastJumpTime = Level.TimeSeconds;
    }

}
simulated function float ChargeBar()
{
    // Clamp to 0.999 so charge bar doesn't blink when maxed
	if (Level.TimeSeconds - JumpDelay < LastJumpTime)
        return (FMin((Level.TimeSeconds - LastJumpTime) / JumpDelay, 0.999));
    else
		return 0.999;
}
simulated Event Tick(float DeltaTime)
{

	Super.Tick(DeltaTime);
	 JumpCountdown -= DeltaTime;
     CheckJumpDuck();

     if(DoBikeJump != OldDoBikeJump)
	{
		JumpCountdown = JumpDuration;
        OldDoBikeJump = DoBikeJump;
	}
	if(!bDoStuntInfo && bVehicleOnGround)
		LastOnGroundTime = Level.TimeSeconds;
	ReallyInAir = (Level.TimeSeconds - LastOnGroundTime) > 0.2;
}

               simulated event UpdateVehicle(float DeltaTime)
{
	Super.UpdateVehicle(DeltaTime);
}

simulated event KImpact(Actor actor, vector Pos, vector ImpactVel, vector ImpactNorm)
{
	Super.KImpact(actor, Pos, ImpactVel, ImpactNorm);
}


function KDriverEnter(Pawn P)
{
	super.KDriverEnter(P);
	Weapons[1].bActive = True;
    if ( StartUpSound != None )
        PlaySound(StartUpSound, SLOT_None, 2.0);
}

function bool ImportantVehicle()
{
	return true;
}

simulated function Destroyed()
{
	super.Destroyed();
}


simulated function AttachDriver(Pawn P)
{
    super.AttachDriver(P);

   //Set Bones so Driver Looks Like its Riding a Cycle
    //Head
    NeckDrive.Yaw=-10000;
    P.SetBoneRotation('Bip01 Head',NeckDrive);
    //Arms
    //LeftArm
    ArmDriveL.Yaw=-5000;
    ArmDriveL.Pitch=-10000;
    P.SetBoneRotation('Bip01 L UpperArm',ArmDriveL);
    //RightArm
    ArmDriveR.Yaw=-5000;
    ArmDriveR.Pitch=10000;
    P.SetBoneRotation('Bip01 R UpperArm',ArmDriveR);
      //Thighs
      //Left Thigh
    ThighDriveL.Pitch=-10000;
    P.SetBoneRotation('Bip01 L Thigh',ThighDriveL);
    //Right Thigh
    ThighDriveR.Pitch=10000;
    P.SetBoneRotation('Bip01 R Thigh',ThighDriveR);
     //Calfs
     //Left Calf
    CalfDriveL.Pitch=6500;
    CalfDriveL.Yaw=-6500;
    P.SetBoneRotation('Bip01 L Calf',CalfDriveL);
    //Right Calf
    CalfDriveR.Pitch=-6500;
    CalfDriveR.Yaw=-6500;
    P.SetBoneRotation('Bip01 R Calf',CalfDriveR);
}
function bool KDriverLeave(bool bForceLeave)
{
   //Return Bones back to good Position
   //Head
    Driver.SetBoneRotation('Bip01 Head');
    //Arms
    //LeftArm
    Driver.SetBoneRotation('Bip01 L UpperArm');
    //RightArm
    Driver.SetBoneRotation('Bip01 R UpperArm');
    //Thighs
    //Left Thigh
    Driver.SetBoneRotation('Bip01 L Thigh');
    //Right Thigh
    Driver.SetBoneRotation('Bip01 R Thigh');
    //Calfs
    //Left Calf
    Driver.SetBoneRotation('Bip01 L Calf');
    //Right Calf
    Driver.SetBoneRotation('Bip01 R Calf');
    Super.KDriverLeave(True);
    return True;
}

simulated function NewWeapon()
{
     local int i;


     // Destroy the weapons
    if (Role == ROLE_Authority)
    {
	    for(i=0;i<Weapons.Length;i++)
        	Weapons[i].Destroy();
    // Spawn the Driver Weapons
        for(i=0;i<DriverWeapons.Length;i++)
        {
             DriverWeapons[i].WeaponClass=BikeWeaponClass;
            // Spawn Weapon
            Weapons[i] = spawn(DriverWeapons[i].WeaponClass, self,, Location, rot(0,0,0));
            AttachToBone(Weapons[i], DriverWeapons[i].WeaponBone);
            if (!Weapons[i].bAimable)
                Weapons[i].CurrentAim = rot(0,32768,0);

        }

        if(Weapons[0].IsA('F_BikeIonPlasma_Weapon'))
           {
            AttachToBone(Weapons[0],'PassAttach');
            SetActiveWeapon(0);
            bKeyVehicle=true;
            Weapons[1].bHidden=true;

           }
      }
        if (Role == ROLE_Authority)
	       {
            if (Weapons.Length == 2 && ONSLinkableWeapon(Weapons[0]) != None)
            FragEdSpecialBike_Weapons(Weapons[0]).ChildWeapon = Weapons[1];
           }
}

simulated function PostNetBeginPlay()
{
     local int i;
    Super.PostNetBeginPlay();

    if(Level.NetMode != NM_DedicatedServer && bHeadLight)
	{
		HeadlightCorona.Length = HeadlightCoronaOffset.Length;

		for(i=0; i<HeadlightCoronaOffset.Length; i++)
		{
			HeadlightCorona[i] = spawn( class'ONSHeadlightCorona', self,, Location + (HeadlightCoronaOffset[i] >> Rotation) );
			HeadlightCorona[i].SetBase(self);
			HeadlightCorona[i].SetRelativeRotation(rot(0,0,0));
			HeadlightCorona[i].Skins[0] = HeadlightCoronaMaterial;
			HeadlightCorona[i].ChangeTeamTint(Team);
			HeadlightCorona[i].MaxCoronaSize = HeadlightCoronaMaxSize * Level.HeadlightScaling;
		}

		if(HeadlightProjectorMaterial != none && bHeadLightProjector)
		{
			HeadlightProjector = spawn( class'ONSHeadlightProjector', self,, Location + (HeadlightProjectorOffset >> Rotation) );
			HeadlightProjector.SetBase(self);
			HeadlightProjector.SetRelativeRotation( HeadlightProjectorRotation );
			HeadlightProjector.ProjTexture = HeadlightProjectorMaterial;
			HeadlightProjector.SetDrawScale(HeadlightProjectorScale);
			HeadlightProjector.CullDistance	= ShadowCullDistance;
		}
	}

    if (Role == ROLE_Authority)
	{

        if (Weapons.Length == 2 && ONSLinkableWeapon(Weapons[0]) != None)
            FragEdSpecialBike_Weapons(Weapons[0]).ChildWeapon = Weapons[1];
   }
}

function DriverLeft()
{
    Super.DriverLeft();
    if (ShutDownSound != None)
        PlaySound(ShutDownSound, SLOT_None, 0.5);
    Weapons[1].bActive = False;
}

function VehicleCeaseFire(bool bWasAltFire)
{
    if(bWeaponIsFiring==true && Weapons[0].IsA('F_BikeGatGun') && F_BikeGatGun(Weapons[ActiveWeapon]) != None)
       {
        F_BikeGatGun(Weapons[ActiveWeapon]).CeaseFire(Controller);
        if (Weapons.Length == 2 && ONSLinkableWeapon(Weapons[0]) != None)
            F_BikeGatGun(Weapons[1]).CeaseFire(Controller);
       }
    Super.VehicleCeaseFire(bWasAltFire);

    if (bWasAltFire && Weapons[0].IsA('F_BikeIonPlasma_Weapon') && F_BikeIonPlasma_Weapon(Weapons[ActiveWeapon]) != None)
        F_BikeIonPlasma_Weapon(Weapons[ActiveWeapon]).CeaseAltFire();


}
defaultproperties
{

    //BikeWeaponClass=class'BattleBikes.F_BikeGatGun'
    RedSkin=Texture'BattleBike_Tex.Bike78SkinB'
    RedSkinB=Texture'BattleBike_Tex.Bike78SkinA'
    BlueSkin=Texture'BattleBike_Tex.Bike78SkinBBlue'
	BlueSkinB=Texture'BattleBike_Tex.Bike78SkinABlue'
     JumpSound=Sound'ONSVehicleSounds-S.HoverBike.HoverBikeJump05'
     JumpDuration=0.220000
     JumpForceMag=100.000000
     JumpDelay=3.000000
     MaxDesireability=0.6
     LocalRollDamping=30.000000
     LocalPitchDamping=8.000000
     AntiRollStiffness=30.000000
     BikeTurnDamping=30.000000
     ObjectiveGetOutDist=1500.000000
     DriverWeapons(0)=(WeaponClass=Class'BattleBikes.F_BikeGatGun',WeaponBone="WeapAttachR")
     DriverWeapons(1)=(WeaponClass=Class'BattleBikes.F_BikeGatGun',WeaponBone="WeapAttachL")
     PassengerWeapons(0)=(WeaponPawnClass=Class'BattleBikes.BikePassenger',WeaponBone="PassAttach")
     BrakeLightOffset(0)=(X=-66.00,Y=5.0,Z=34.000000)
     BrakeLightOffset(1)=(X=-66.00,Y=-5.0,Z=34.000000)
     bMakeBrakeLights=True
     BrakeLightMaterial=Texture'EpicParticles.Flares.FlashFlare1'
     HeadlightCoronaOffset(0)=(X=58.000000,Y=0.000000,Z=34.000000)
     HeadlightCoronaMaterial=Texture'EpicParticles.Flares.FlashFlare1'
     HeadlightCoronaMaxSize=100.000000
     HeadlightProjectorMaterial=Texture'VMVehicles-TX.RVGroup.RVprojector'
     HeadlightProjectorOffset=(X=60.000000,Z=34.000000)
     HeadlightProjectorRotation=(Pitch=-1000)
     HeadlightProjectorScale=0.300000
     BikeJumpForce=1800.000000
     WheelAdhesion=0.5
     WheelSoftness=0.04000
     WheelPenScale=1.800000
     WheelPenOffset=-0.010000
     WheelRestitution=0.150000
     WheelInertia=0.100000
     WheelLongFrictionFunc=(Points=(,(InVal=100.000000,OutVal=1.000000),(InVal=200.000000,OutVal=0.900000),(InVal=10000000000.000000,OutVal=0.900000)))
     WheelLongSlip=0.001000
     WheelLatSlipFunc=(Points=(,(InVal=30.000000,OutVal=0.009000),(InVal=45.000000),(InVal=10000000000.000000)))
     WheelLongFrictionScale=1.300000
     WheelLatFrictionScale=7.0//2.000000
     WheelHandbrakeSlip=0.010000
     WheelHandbrakeFriction=0.100000
     WheelSuspensionTravel=15.000000
     WheelSuspensionOffset=-1.500000
     WheelSuspensionMaxRenderTravel=10.000000
     FTScale=0.030000
     MinBrakeFriction=4.000000
     MaxSteerAngleCurve=(Points=((OutVal=25.000000),(InVal=1500.000000,OutVal=8.000000),(InVal=25000.000000)))
     TorqueCurve=(Points=((OutVal=10.000000),(InVal=200.000000,OutVal=17.000000),(InVal=1500.000000,OutVal=20.000000),(InVal=3800.000000)))
     GearRatios(0)=-0.400000
     GearRatios(1)=0.400000
     GearRatios(2)=0.650000
     GearRatios(3)=0.850000
     GearRatios(4)=1.100000
     TransRatio=0.150000
     ChangeUpPoint=3000.000000
     ChangeDownPoint=1000.000000
     LSDFactor=1.000000
     EngineBrakeFactor=0.000100//0.000100
     EngineBrakeRPMScale=0.100000
     MaxBrakeTorque=200//75.000000
     SteerSpeed=120.00//100.000000
     TurnDamping=35.00
     StopThreshold=100.0//100.000000
     HandbrakeThresh=200.00//200.000000
     EngineInertia=0.150000
     IdleRPM=500.000000
     EngineRPMSoundRange=12000.000000
    // SteerBoneName="Steer"
     SteerBoneAxis=AXIS_Z
     SteerBoneMaxAngle=90.000000
     OutputBrake=1.000000
     Gear=1
     RevMeterScale=4000.000000

     DaredevilThreshInAirSpin=180.000000
     DaredevilThreshInAirTime=5.000000
     DaredevilThreshInAirDistance=60.000000
     bAllowAirControl=True
     bAllowBigWheels=True
     AirTurnTorque=35.000000
     AirPitchTorque=55.000000
     AirPitchDamping=35.000000
     AirRollTorque=50//35.000000
     AirRollDamping=35.000000
     MinAirControlDamping=0.200000
     bHasAltFire=true

     SoundVolume=180
     IdleSound=Sound'BattleBike_Snds.BikeEngine1'//Sound'Cyclone_Snds.BikeEngIdle'
     StartUpSound=Sound'BattleBike_Snds.BikeStart'
     ShutDownSound=Sound'BattleBike_Snds.BikeOff'
     StartUpForce="RVStartUp"
     DestroyedVehicleMesh=StaticMesh'BattleBikes_ST.FragedSpecialDesST'
     DestructionEffectClass=Class'Onslaught.ONSSmallVehicleExplosionEffect'
     DisintegrationEffectClass=Class'BattleBikes.BikeVehDeathRV'
     DisintegrationHealth=-25.000000
     DestructionLinearMomentum=(Min=200000.000000,Max=300000.000000)
     DestructionAngularMomentum=(Min=100.000000,Max=150.000000)
     DamagedEffectOffset=(X=0.000000,Y=0.000000,Z=10.000000)
     bEjectPassengersWhenFlipped=False
     ImpactDamageMult=0.001000
     Begin Object Class=SVehicleWheel Name=SVehicleWheel6
         SteerType=VST_Steered
         BoneName="FrontWheel"
         WheelRadius=21.500000
         SupportBoneName="RearStrut"
         BoneRollAxis=AXIS_Y
         SupportPivotDistance=8.198524
     End Object
     Wheels(0)=SVehicleWheel'BattleBikes.FragEdSpecial.SVehicleWheel6'

     Begin Object Class=SVehicleWheel Name=SVehicleWheel7
         ChassisTorque=1.000000
         bPoweredWheel=True
         //bHandbrakeWheel=True
         BoneName="RearWheel"
         WheelRadius=21.500000
         SupportBoneName="FrontStrut"
         //SupportBoneAxis=AXIS_X
         BoneRollAxis=AXIS_Y
         SupportPivotDistance=-9.545372
     End Object
     Wheels(1)=SVehicleWheel'BattleBikes.FragEdSpecial.SVehicleWheel7'

     VehicleMass=2.500000
     bDrawDriverInTP=True
     //bFollowLookDir=True
     bDrawMeshInFP=True
     bTeamLocked=False
     bHasHandbrake=True
     bSeparateTurretFocus=True
     bDriverHoldsFlag=true
     bCanCarryFlag=true
     DrivePos=(X=23.000000,Z=52.000000)
     DriveRot=(Pitch=-14000)
     ExitPositions(0)=(Y=-50.000000,Z=25.000000)
     ExitPositions(1)=(Y=50.000000,Z=25.000000)
     ExitPositions(2)=(X=-120.000000,Z=25.000000)
     ExitPositions(3)=(X=120.000000,Z=25.000000)
     EntryRadius=160.000000
     TPCamDistance=375.000000
     CenterSpringForce="SpringONSSRV"
     FPCamPos=(X=20.000000,Y=0.000000,Z=50.000000)
     TPCamLookat=(X=20.000000,Z=50.000000)
     TPCamWorldOffset=(Z=100.000000)
     DriverDamageMult=0.300000
     VehiclePositionString="on a FragED Special"
     VehicleNameString="FragEd Special"
     RanOverDamageType=Class'Onslaught.DamTypeHoverBikeHeadshot'
     CrushedDamageType=Class'Onslaught.DamTypeHoverBikePancake'
     HornSounds(0)=Sound'ONSVehicleSounds-S.Horns.Horn06'
     HornSounds(1)=Sound'ONSVehicleSounds-S.Horns.Dixie_Horn'
     GroundSpeed=940.000000
     HealthMax=175.000000
     Health=175.0
     bReplicateAnimations=True
     Mesh=SkeletalMesh'BattleBikes_Anim.FragEdSpecMesh'
     CollisionRadius=100.00//64.000000
     CollisionHeight=40.000000
     DrawScale=0.6
     Begin Object Class=KarmaParamsRBFull Name=KarmaParamsRBFull2
         KInertiaTensor(0)=1.000000
         KInertiaTensor(3)=3.000000
         KInertiaTensor(5)=3.000000
         KCOMOffset=(Z=-0.900000)
         KLinearDamping=0.050000
         KAngularDamping=0.050000
         KStartEnabled=True
         bKNonSphericalInertia=True
         KActorGravScale=1.200000
         bHighDetailOnly=False
         bClientOnly=False
         bKDoubleTickRate=True
         bDestroyOnWorldPenetrate=True
         bDoSafetime=True
         StayUprightStiffness=1.000000
         KFriction=0.600000
         KImpactThreshold=700.000000
     End Object
     KParams=KarmaParamsRBFull'BattleBikes.FragEdSpecial.KarmaParamsRBFull2'

}

